/**
 * La classe Matrices represente l'ensembles des matrices de travail.
 * @author Aude Liefooghe
 */

#ifndef Matrices_h
#define Matrices_h

#include <stack>
#include <map>
#include <list>
#include <math.h>
#include "Distribution.h"
#include "Matrice.h"


/**
Permet de classer l'ensemble des matrices de travail en taille decroissante
*/
struct more_mat : public binary_function < Matrice *, Matrice *, bool >
{
  bool operator() ( Matrice * m1, Matrice * m2 )
  {
    return m1->getTaille() > m2->getTaille();
  }
};


class Matrices
{
  /// La taille de la plus grande matrice
  int tailleMaxMatrice;
  /// La taille des la plus petite matrice
  int tailleMinMatrice;
  /// La p-valeur
  int pValeur;
  /// Le pseudo compte
  int psc;
  /// Le nombre de matrices
  int nbMatrice;
  /// Le tableau de comptage des tailles des matrices
  int * tailleMatrices;
  /// Le tableau de comptage des matrices de taille superieur
  int * tabNbMatriceTailleSup;
  /// Le tableau des d�alages permettant un alignement optimal entre les matrices
  int ** tabDifDec;
  /// Le tableau des seuils de l'ensemble des matrices
  int * seuils;
  /// La precision des scores des matrices en nombre entier
  int precision;
  /// Le chemin d'acces au repertoire contenant le fichier de la liste des matrices
  string chemin;
  /// Le nom du fichier contenant la liste des matrices
  string fichierMatrices;
  /// Le tableau des noms des fichiers des matrices
  string * fichiersMatrices ;
  /// Le tableau de l'ensemble des matrices
  Matrice * * matriceMatrices;
  /// Le tableau de la probabilite de recalcul, lors de l'estimation des hits d'une matrice par une autre, pour l'alignement optimal des matrices
  float * * tabDifVal;
  /// La proportion de G et de C pour laquelle les scores sont calcule
  float pourGC;
  /// Le tableau de correspondance entre le nom de la matrice (qui est aussi le nom de sont fichier) et sont indice dans le tableau des matrices
  map < string, int > tbCorres;

public:
  int getPrecision();
    int getPValeur();
  int getPsc();
  /**
  Constructeur de la classe
  Construit le tableau des matrices a partir d'un fichier
  @param f Le fichier contenant la liste des fichiers des matrices
  @param pgc La proportion de G et de C pour laquelle les scores sont calcule
  @param p Le pseudo compte
  @param pVal La p-valeur
  */
  Matrices( string f, int prec, float pgc = 0.5, int p = 1, int pVal=5,  bool b=true   );
  /**
   Constructeur de la classe
   Construit le tableau des matrices
   @param **m le tableau des matrices
  @param Le nombre de matrices
   @param pgc La proportion de G et de C pour laquelle les scores sont calcule
   @param p Le pseudo compte
  @param pVal La p-valeur
   */
  Matrices( Matrice * * m, int nM, float pgc = 0.5, int p = 1, int pVal=5 );
  /**
  @return Le nombre de matrices
  */
  int getNbMatrice();
  /**
  @return La taille de la plus grande matrice
  */
  int getTailleMaxMatrice();
    /**
  @return La taille de la plus petite matrice
  */
  int getTailleMinMatrice();
  /**
  @return Le tableau de comptage des matrices de taille superieur
  */
  int * getTabNbMatriceTailleSup();
  /**
  @param i taille de matrice
  @return le nombre de matrices de taille sup�ieur �i
  */
  int getTabNbMatriceTailleSup( int i );
  /**
  @param i indice de la matrice
  @return La matrice d'indice i
  */
  Matrice * getMatrice( short i );
  
  float getpGC();
  /**
  @return Le tableau de la probabilite de recalcul, lors de l'estimation des hits d'une matrice par une autre, pour l'alignement optimal des matrices
  */
  float * * getTabDifVal();
  /**
  @param i indice de matrice
  @param j indice de matrice
  @return La probabilite de recalcul, lors de l'estimation des hits de la matrice d'indice j par la matrice d'indice i, pour leur alignement optimal
  */
  float getTabDifVal( short i, short j );
  /**
  @param i indice de matrice
  @param j indice de matrice
  @return Le decalage entre la matrice d'indice j et la matrice d'indice i pour leur alignement optimal
  */
  int getTabDifDec( short i, short j );
  /**
  @param i indice de matrice
  @return Le seuil de la matrice d'indice i
  */
  int getSeuil( int i );

  /**
  @param n le nom d'une matrice
  @return l'indice de la matrice de nom n
  */
  int getCorres( string n );

private:

  /**
  Calcule Le tableau des seuils de l'ensemble des matrices
  */
  void calcSeuil();
  /**
  Calcule Le tableau des d�alages permettant un alignement optimal entre les matrices et le tableau de comptage des matrices de taille superieur
  */
  void DifMatrices();
  /**
  Cree le tableau de correspondance entre le nom de la matrice (qui est aussi le nom de sont fichier) et sont indice dans le tableau des matrices
  */
  void createTbCorres();
  /**
  @param k indice de matrice
  @param l indice de matrice
  @param decalage decalage de la matrice d'indice l par rapport a la matrice d'incide k
  @param b vrai si la proportion de G et C est cossideree equi distribue
  @return la probabilite de recalcul, lors de l'estimation des hits de la matrice d'ince k par la matrice d'indice l, pour un decalage entre les matrice de decalage
  */
  int DifPosMaxCons( int k, int l, int decalage, bool b = 1 );
  /**
  Remplie le tableau des matrices en creent l'ensemble des matrices
  */
  void remplitTab();
  /**
  Calcule l'ensemble des parametres concernant les tailles des matrices
  */
  void param();
  /**
  Lit le fichier contenant la liste des fichiers des matrices et cree le tableau contenant les noms des fichiers des matrices
  */
  void lectFicMatrices();
};

#endif

