/**
 * La classe StructMatrice represente un index des scores de l'ensemble des matrices de travail
 * @author Aude Liefooghe
 */
 
 
#ifndef StructMatrice_h
#define StructMatrice_h

#include <stack>
#include <list>
#include <math.h>

#include "Matrices.h"
#include "Couple.h"
#include "Decoupage.h"


class StructMatrice
{
   /// La structure : le tableau d'index des scores de l'ensemble des matrices indexes par l'ensembles des mots, 
   /// divise en sous-tableaux donc en sous-mots et en sous-scores par espace memoire limite
   typedef int ** tab;
   tab ** structMatrices;
   /// Espace memoire dedie  la structure en nombre d'entier
   unsigned long nbCaseMax;
   /// Le tableau qui stock le nombre de mots  indexer pour une taille de mots stocke dans le tableau decoupage
   unsigned long * taille;
   /// (Variable de travail)
   unsigned long * taille2;
   /// Le nombre de sous-tableaux de la structure 
   int nbTab;
   /// Le nombre de matrice prise en compte dans la structure
   int nbMatriceStruct;
   /// Le tableau des tailles de mots des sous-tableaux de la structure
   int * decoupage;
   ///(Variable de travail)
   int * decoupage2;
   /// Le tableau des tailles de mots cumules des sous-tableaux de la structure
   int  * decoupageCumul;
   /// L'ensemble des matrices de travail
   Matrices* matrices;
   /// Le tableau des nombres d'acces  un sous-tableau de la structure selon sa position de depart 
   double *nbAcess;
   /// Le tableau de l'espace mmoire occupe en nombre d'entier par un sous-tableau de la structure 
   /// selon sa position de depart
   long ** nbCase ;

 public:
   /**
      Constructeur de la classe
      Construit la structure pour l'ensemble des matrices e travail
      @param m L'ensemble des matrices de travail
      @param mo l'espace memoire ddiee a la structure
   */
   StructMatrice( Matrices* m, unsigned long mo);
   /**
      @return Le nombre de matrice prise en compte dans la structure
   */
   int getNbMatriceStruct();
   /**
      @return La taille mots cumules du ieme sous-tableau de la structure
      @param i L'indice du sous-tableau
   */
   int getDecoupageCumul(int i);
   /**
      @return Le nombre de sous-tableaux
   */
   int getNbTab();
   /**
      @return La taille mots du ieme sous-tableau de la structure
      @param i L'indice du sous-tableau
   */
   int getDecoupage(int i);
   /**
      @return Les sous-scores de l'ensemble des matrices correspondant au sous-tableau i et au sous-mar cod par j
      @param i L'indice du sous-tableau
   */
   int ** getStructMatrice( int i, unsigned long j );

 private:
   /**
      Ajoute la matrice i a la structure
      @param i L'indice de la matrice  
   */
   void addMatrice( int i );
   /**
      Ajoute la matrice i au sous-tableau partie de la structure 
      @param j L'indice de la matrice  
      @param partie L'indice du sous tableau
      @param nbTabUtile Le nombre de sous-tableaux utile a la matrice j
   */
   void addMatriceDecoupage( int j, int partie, int nbTabUtile );

   void rechercheCombin();
   /**
      Construit le sous-tableau j aux 2 dimentions nbMat et t
      @param j L'indice du sous-tableau
      @param nbMat Le nombre de matrice indices du sous-tableau j
      @param t Le nombre de mots correspondant  la taille de decoupage du sous-tableau j
   */
   void constTab( int j, int nbMat, int t );
   /**
      @return Le nombres d'acces au sous-tableau de la structure de position de depart a
   */
   double calcNbAcess(int a);
   /**
      L'espace mmoire occupe en nombre d'entier par un sous-tableau de la structure 
      allant de la position a a la position b 
   */
   unsigned long calcNbCase(int a, int b);

   /**
      Stoke dans les tableaux le nombres d'acces au sous-tableau de la structure de position de depart a et l'espace mmoire occupe en nombre d'entier par un sous-tableau de la structure 
      allant de la position a a la position b et 
   */
   void calcTranches();

   /**
      Recherche les tailles optimales de dcoupage de la structure
   */
   Decoupage  calcDecouage(int nbTab, int k, long nbC);
};
#endif


