#include <string>
#include <time.h>
#include <getopt.h>
#include "StructMatrice.h"
#include "Sequence.h"
#include "Metier.h"
#include "Metier2.h"
#include "Noyau.h"

#include "Clusterisation.h"
#include "Cluster2.h"

struct more_clust : public binary_function < Cluster2 *, Cluster2 *, bool >
{
  bool operator() ( Cluster2 c1, Cluster2 c2 )
  {
    return c1.getMoyenne()->getTaille() >  c2.getMoyenne()->getTaille();
  }
};

using namespace std;

Matrices * Cluster2 :: matrices = NULL;

Metier * clust(Matrices * mats, int precision, float pGC, int psc, int pVal, double prop, int pourGC, string res, int size, bool loc)
{
  Cluster2::setMatrices(mats);
  Clusterisation clust;
  clock_t c1, c2;
  c1=clock();
  vector <Cluster2> listClust= clust.make_cluster(mats->getTabDifVal(),mats->getNbMatrice(), prop, mats);
  c2=clock();
  //cout << "clock time of cluterisation : " << difftime(c2, c1)/CLOCKS_PER_SEC << " second(s)." << endl;
  for ( int l = 0; l < listClust.size(); l++ )
  {
    long double tb[listClust[l].getNbMat()];
    for ( short j = 0; j < listClust[l].getNbMat(); j++ )
    {
      tb[j] = 0;
      for ( short i = 0; i < listClust[l].getNbMat(); i++ )
      {
        tb[j] += mats->getTabDifVal( listClust[l].getMat( j ), listClust[l].getMat( i ));
      }
    }
    if ( listClust[l].getNbMat() > 0 )
    {
      long double * ad = min_element( tb, tb + ( listClust[l].getNbMat() ) );
      listClust[l].setNoyau( listClust[l].getMat( ad - tb ) );
    }
  }
  short nbClust = listClust.size();
  Matrice ** matNoy2;
  matNoy2 = new Matrice * [nbClust];
  for ( int i = 0; i < nbClust; i++ )
  {
    if (listClust[i].getNbMat()>1)
    {
      short s0 = listClust[i].getNoyau();
      vector <Matrice*> m;
      for (int j=0; j<listClust[i].getNbMat(); j++)
      {
        short s1 = listClust[i].getMat(j);
        m.push_back( mats->getMatrice(s1) );
        m[j]->setDec(mats->getTabDifDec(s0, s1));
      }
      Noyau n(m, precision, psc, pVal, pGC);
      listClust[i].setMoyenne(n.getMatriceMoy());
    }
    else
    {
      listClust[i].setMoyenne(listClust[i].getNoyau2());
    }
  }
  sort( listClust.begin(), listClust.end(), more_clust() );
  for ( int i = 0; i < nbClust; i++ )
  {
    matNoy2[i]=listClust[i].getMoyenne();
  }
  Matrices * matsN2 = new Matrices( matNoy2, nbClust, pGC, psc, pVal);
  StructMatrice * smN = new StructMatrice ( matsN2, size );
  return new Metier( smN, listClust, mats, matsN2, pourGC, res, loc );
}



void usage()
{
  cerr << "./TFMscan -m <matrices_file> -s <sequence_file> [-p significance_threshold] [-l]  [-g gc_percent] [-M memory_size] [-o <output_file>][-c 0|1|2][-e error]\n\nOptions\n\n-m  <matrices_file>\n<matrices_file> must contain a list of files, each file containing the description of a Position Weight Matrix.  The synatx is as follows : the first line is the directory where matrix files are located, the other lines are the relative pathnames of each matrix file. To see an example, consult the Samples directory (*.mat files).\n\n-s <sequence_file>\n <sequence_file> contains the DNA sequence, in Fasta format.\n\n-p significance_threshold\nP-value=ln(significance_threshold). This is a positive natural number.  By default, significance_thereshold=8, which corresponds to P-value=3.35e-4.\n\n-g gc-percent\nThis option specifies the GC percentage used in the Bernoulli backgound model for the computation of the score. gc_percent is a decimal number between 0.3 and 0.7.  By default, the GC percentage is computed on the whole input sequence.\n\n-l\nThis option is another way to specify the GC percentage used in the background model.  It allows the program to use variable GC percentage. It is computed locally along the input sequence with sliding windows (200 bases). This option may give more relevant results, but it is slower. Options -c and -l are mutually incompatible.\n\n-M memory_size\nThe maximal amount of memory in MB that the software uses for the index table. Default value is 256MB.  Be careful when using a large value for this parameter: if it exceeds the memory capacity of your computer, it may dramatically slow down the program or cause it to fail.\n\n-o <ouput_file>\nFile that contains the results of TFM-Scan. Each matrix occurrence is given by its location on the sequence, the name of the matrix, and the score of the occurrence.  By default, results are displayed on the standard output.\n\n-c 0|1|2\nTFMscan provides three variants of the search algorithm:\n0 : main algorithm; \n1 : gives the same result as option -c O, but it uses a lossless filtering with matrix clustering. This option may speed up the search;\n2 : Uses an approximate search algorithm that gathers all similar matrices and all similar occurrences into clusters. This option may speed up the search;\nDefault value is 0. Options 1 and 2 are appropriate when dealing with families of similar matrices (for example, a set of matrices corresponding to the same transcription factor).\n\n-e error\nThis parameter is the maximal error rate for the approximate algorithm (comprised between 0 and 1).  So it is usefull only with option -c 2, otherwise it is ignored. Default value is 0.3."<< endl;
  exit(-1);
} 


int main( int argc, char * * ARGV )
{

  int c;
  extern char *optarg;
  string matrices="", sequence="", results="";
  int memoire=256, pVal=8, option=0;
  float pGC=0, dist=0.3;
  bool local=false;
  while ((c = getopt(argc,ARGV,"m:s:p:g:lM:o:c:e:")) != EOF)
  {

    switch (c)
    {
       case 'l' :
          local = true;
          break;
	  
    case 'g' :
      if (strlen(optarg) !=0 && optarg[0]!= '-')
      {
        pGC= atof(optarg);
          if (pGC >0.7 || pGC<0.3)
          {
            cerr << "ERROR : The GC percentage is a decimal number between 0.3 and 0.7.\n" << endl;
            usage();
          }
      }
      else
      {
        cerr << "ERROR : bad use of -g option.\n" << endl;
        usage();
      }
      break;
    case 'm':
      if (strlen(optarg) !=0 && optarg[0]!= '-')
      {
        matrices = optarg;
      }
      else
      {
      cerr << "ERROR : bad use of -m option.\n" << endl;
        usage();
      }
      break;
    case 's':
      if (strlen(optarg) !=0 && optarg[0]!= '-')
      {
        sequence = optarg;
      }
      else
      {
      cerr << "ERROR : bad use of -s option.\n" << endl;
        usage();
      }
      break;
    case 'o':
      if (strlen(optarg) !=0 && optarg[0]!= '-')
      {
        results = optarg;
      }
      else
      {
      cerr << "ERROR : bad use of -o option.\n" << endl;
        usage();
      }
      break;
    case 'M':
      if (strlen(optarg) !=0 && optarg[0]!= '-')
      {
        memoire = atoi(optarg);
      }
      else
      {
      cerr << "ERROR : bad use of -M option.\n" << endl;
        usage();
      }
      break;
    case 'p':
      if (strlen(optarg) !=0 && optarg[0]!= '-')
      {
        pVal = atoi(optarg);
      }
      else
      {
      cerr << "ERROR : bad use of -p option.\n" << endl;
        usage();
      }
      break;
    case 'c':
      if (strlen(optarg) !=0 && optarg[0]!= '-')
      {
        option = atoi(optarg);
	if (option <0 || option >2) {
	cerr << "ERROR : bad use of -c option.\n" << endl;
        usage();
	}
      }
      else
      {
      cerr << "ERROR : bad use of -c option.\n" << endl;
        usage();
      }
      break;
    case 'e':
      if (strlen(optarg) !=0 && optarg[0]!= '-')
      {
        dist = atof(optarg);
      }
      else
      {
      cerr << "ERROR : bad use of -e option.\n" << endl;
        usage();
      }
      break;
    default:
      usage();
    }

  }
  if (matrices=="" || sequence=="")
  {
  cerr << "ERROR : Options -m and -s are obligatory.\n" << endl;
    usage();
  }

  if (local && pGC != 0)
  {
   cerr << "ERROR : Options -c and -l are mutually incompatible.\n" << endl;
    usage();
  }

  Sequence * seq = new Sequence(sequence);
  if (pGC==0) pGC = seq->getpGC();
  if (pGC < 0.3) pGC = 0.3;
  if (pGC > 0.7) pGC = 0.7;
  int psc = 1;
  int pourGC = ((int)(pGC*100)/5)-6;
  int precision = 10;
  clock_t c1, c2;
  Metier *metier;
  Metier2 *metier2;
  Matrices * mats = new Matrices(matrices, precision, pGC, psc, pVal);

  cout << endl;
    cout <<  "Sequence file : " << sequence << endl;
    cout << "Matrix file : " << matrices << " (" << mats->getNbMatrice() << " matrices)" << endl;
    cout << "Exact algorithm : ";
  if (option ==0)
  	cout << "yes" << endl;
  else 
  	cout << "no (error rate : " << dist <<")" << endl;
   cout <<  "P-value : e^-" <<pVal<<" ("<< pow( ( long double )exp(1.0), -pVal ) << ")" << endl;
   cout << "GC-percent : ";
   if (local) 
   	cout << "local" << endl;
   else
   	cout << pGC << endl;
cout << endl;
	
 if (results !=""){
ofstream fs( results.data(), ios_base::out );
    fs <<  "Sequence file : " << sequence << endl;
    fs << "Matrix file : " << matrices << " (" << mats->getNbMatrice() << " matrices)" << endl;
    fs << "Exact algorithm : ";
  if (option ==0)
  	fs << "yes" << endl;
  else 
  	fs << "no (error rate : " << dist <<")" << endl;
   fs <<  "P-value : e^-" <<pVal<<" ("<< pow( ( long double )exp(1.0), -pVal ) << ")" << endl;
   fs << "GC-percent : ";
   if (local) 
   	fs << "local" << endl;
   else
   	fs << pGC << endl;
fs<<endl;
}
	
  if(option==0)
  {
    StructMatrice * sm = new StructMatrice( mats, memoire );
    metier2 = new Metier2( sm, mats, pourGC, results, local);
  }
  else
  {
    metier = clust(mats, precision, pGC, psc, pVal, dist, pourGC, results, memoire, local );
  }
  if(option==0)
  {
cout << "Computing occurrences... Done" << endl;
    metier2->setSequence(seq);
    c1= clock();
    metier2->lesHits();
    c2= clock();
  }
  if(option==1)
  {
    metier->setSequence(seq);
cout << "Computing occurrences... Done" << endl;
    c1= clock();
    metier->lesHitsExact();
    c2= clock();
  }
  if(option==2)
  {
    metier->setSequence(seq);
    cout << "Computing occurrences... Done" << endl;
    c1= clock();
    metier->lesHits2();
    c2= clock();
  }
  if (results!="") 
  	cout << "Results are available in " << results << endl;

cout << "Elapsed time : " << difftime(c2, c1)/CLOCKS_PER_SEC << " second(s)." << endl;
  return 0;
}


